#!/bin/bash

# ------------------------------------------------------------
# install the initial VM for 
#
#       OpenSecurity V0.1
#
# This has been originally a Windows only BAT file.
#
# ... but coding this makes your head hurt and
# supporting this "technology" any further by adding
# software to the world relying on CMD.exe is an act
# against humanity and should be punished by jail.
#
# (C)opyright 2014, AIT Austrian Instiitute of Technology
# ------------------------------------------------------------


# ------------------------------------------------------------
# vars

OPENSECURITY_PORT=8080


# ------------------------------------------------------------
# code


# ------------------------------
# press a key message to the user
# 
function press_a_key() {
    echo "press a key ..."
    read -n 1 X
    echo
}


# ------------------------------
# read a value from the windows regsitry
#
#   $1  ...     full registry key
#   $2  ...     value name
#   stdout      the value found
# 
function reg_read() {

    test -z "${1}" && return
    
    if [ -z "${2}" ]; then
        reg.exe query "${1}"
    else
        reg.exe query "${1}" /v "${2}" | grep "${2}" | sed 's/^.*REG_[A-Z]* *//' 
    fi 
}


# ------------------------------
# turns a windows path into a cygwin path
#
#   $1  ...     windows path
#   stdout      the value found
#
function sanitize_path() {
    test -z "${1}" && return
    echo $(cygpath -u "${1}") 
}


# ------------------------------
# main ...
#

# check if we do have elevated rights
# that is "Run as Administrator" invocation
id -G | grep 544 &> /dev/null
if [ "${?}" != 0 ]; then
    echo "Insufficient privileges."
    echo "Is this script executed with 'Run As Administrator'?"
    press_a_key
    exit 1
fi

# ensure we have /OpenSecurity as install folder
os-install-path.sh

# check OpenSecurity Initial VM Image
#
OSECVM_IMAGE="/OpenSecurity/vm/OsecVM.ova"
if [ ! -f "${OSECVM_IMAGE}" ]; then
    echo "Warning: no OpenSecurity Initial Image found."
    echo "Please download using the OpenSecurity download tool."
    press_a_key
    exit 1
fi

# look up VirtulBox installation
#
VBOX_MANAGER="$(reg_read 'HKLM\Software\Oracle\VirtualBox' 'InstallDir')VBoxManage.exe"
VBOX_MANAGER=$(sanitize_path "${VBOX_MANAGER}")
if [ ! -x "${VBOX_MANAGER}" ]; then
    echo "can't execute VBoxManage.exe - is VirtualBox installed?"
    echo "looked at: "$(cygpath -w ${VBOX_MANAGER})""
    press_a_key
    exit 1
fi

# VirtualBox works on different folders which do
# have admin privileges and sensible data.
# Don't mingle with any User data.
export HOME="$(cygpath.exe -w /OpenSecurity/vm)"
export VBOX_USER_HOME=${HOME}
export USERPROFILE=${HOME}

# enforce VirtualBox to "feel good" by calling a function
# (that is to "warm up" VirtualBox DCOM server ...)
#
VBOX_VERSION=$("${VBOX_MANAGER}" -version)
"${VBOX_MANAGER}" list vms &> /dev/null


# the Security VM disk image
VDISK_IMAGE=$(cygpath -w "/OpenSecurity/vm/SecurityDVM/SecurityDVM.vmdk")

# import VM 
#
"${VBOX_MANAGER}" list vms | grep SecurityDVM &> /dev/null
if [ ! "${?}" = "0" ]; then
    echo "importing VM: ${OSECVM_IMAGE}"
    "${VBOX_MANAGER}" import "$(cygpath -w "${OSECVM_IMAGE}")" --vsys 0 --vmname SecurityDVM --unit 12 --disk "${VDISK_IMAGE}"
else
    echo "found SecurityDVM already present in VBox reusing it."
    echo "if you want a complete new import please remove the VM first."
    press_a_key
    exit 1
fi

# grab VM storage controller and port 
#
VDISK_SETUP=$("${VBOX_MANAGER}" showvminfo SecurityDVM | grep SecurityDVM.vmdk | cut -d ':' -f 1 | tr '(),' '   ')
VDISK_CONTROLLER=$(echo ${VDISK_SETUP} | gawk '{print $1;}')
VDISK_PORT=$(echo ${VDISK_SETUP} | gawk '{print $2;}')
VDISK_DEVICE=$(echo ${VDISK_SETUP} | gawk '{print $3;}')
if [ -z "${VDISK_CONTROLLER}" ]; then
    echo "unable to grab virtual disk controller in VM."
    echo "this shouldn't happen. It's a bug."
    press_a_key
    exit 1
fi

# detach disk image
#
echo "detaching disk image ..."
"${VBOX_MANAGER}" storageattach SecurityDVM --storagectl ${VDISK_CONTROLLER} --port ${VDISK_PORT} --medium none

# turn disk image into writethrough
#
echo "turning disk image into writetrough ..."
"${VBOX_MANAGER}" storageattach SecurityDVM --storagectl ${VDISK_CONTROLLER} --port ${VDISK_PORT} --device ${VDISK_DEVICE} --type hdd --mtype writethrough --medium "${VDISK_IMAGE}" 

# detach disk image
#
echo "detaching disk image ..."
"${VBOX_MANAGER}" storageattach SecurityDVM --storagectl ${VDISK_CONTROLLER} --port ${VDISK_PORT} --medium none

# immutablize disk
#
echo "reattach immutable disk image ..."
"${VBOX_MANAGER}" storageattach SecurityDVM --storagectl ${VDISK_CONTROLLER} --port ${VDISK_PORT} --device ${VDISK_DEVICE} --type hdd --mtype immutable --medium "${VDISK_IMAGE}"

echo "imported initial OsecVM.ova image"

"${VBOX_MANAGER}" list vms


